function beta = betaopt(loglambda0,bgacv,output)
% This function comes from LPS. It has different d1, d2, nhalf1 and nhalf2

global fBasis;
[sampsize nbeta] = size(fBasis);

if bgacv == 1;
% select the smoothing parameter by bgacv
    
%--------------------------------------------------------------------------
% The first grid search
%--------------------------------------------------------------------------
d1=0.2;
d2=0.04;
nhalf1 = 2;
nhalf2 = 4;
ngrid1 = 2*nhalf1 + 1;                  % needs to be odd
ngrid2 = 2*nhalf2;                      % needs to be even
ngrid = ngrid1 + ngrid2;
loglambda = (d1*nhalf1:-d1:-d1*nhalf1) + loglambda0;
k=0;
beta_init = 0.0*ones(nbeta,1);   % use starting point of zero at first
tunescore = zeros(1,ngrid);
loglike = zeros(1,ngrid);
Tlambda = zeros(1,ngrid);
numNonzeros = zeros(1,ngrid);
lambdaok = 0;
nsearch = 1;
while ((~lambdaok)&&nsearch<=3)
    lambdaok = 1;
    tunescore_min = Inf;
    for i=1:ngrid1,
        k=k+1;
        [tunescore(i), beta, loglike(i), Tlambda(i), numNonzeros(i)] = ...
            cv(10^loglambda(i),beta_init,output);
        if tunescore(i) < tunescore_min,
            tunescore_min = tunescore(i);
            loglambda_min = loglambda(i);
            numNonzeros_min = numNonzeros(i);
            beta_best = beta;
            beta_prev = beta_init;
            inx = i;
        end
        beta_init = beta;      % remember starting point for next iteration
    end
    nsearch = nsearch + 1;
    if inx == 1                % if best lambda is the smallest, decrease
        loglambda0 = loglambda0 - (nhalf1 - 1)*d1;
        loglambda = (d1*nhalf1:-d1:-d1*nhalf1) + loglambda0;
    elseif inx == ngrid1       % if best lambda is the biggest, increase
        loglambda0 = loglambda0 + (nhalf1 - 1)*d1;
        loglambda = (d1*nhalf1:-d1:-d1*nhalf1) + loglambda0;
    else
        lambdaok = 1;
    end
end

%--------------------------------------------------------------------------
% Reevaluate on a finer grid of regularization parameters near optimal
% value
%--------------------------------------------------------------------------
loglambda2 = loglambda_min + [nhalf2*d2:-d2:d2, -d2:-d2:-nhalf2*d2];
loglambda1 = [loglambda loglambda2];
loglambda = loglambda1; clear loglambda1 loglambda2
if inx ~=1                    % reset starting point for cv()                  
    beta_init = beta_prev;
else
    beta_init = beta_best;
end
for i = (ngrid1+1:ngrid1+ngrid2)
    k = k + 1;
    [tunescore(i), beta, loglike(i), Tlambda(i), numNonzeros(i)] = ...
        cv(10^loglambda(i),beta_init,output);
    if tunescore(i) < tunescore_min,
        tunescore_min = tunescore(i);
        loglambda_min = loglambda(i);
        numNonzeros_min = numNonzeros(i);
        beta_best = beta;
    end
    beta_init = beta;         % remember starting point for next iteration
end
beta = beta_best;
if output == 1
fprintf('Minimizing reg.param is %8.5f with number of nonzeros %g\n\n', ...
    10^loglambda_min, numNonzeros_min);
end

%--------------------------------------------------------------------------
% Sort the regularization parameters and print summary of results
%--------------------------------------------------------------------------
[tunpSorted, tunpIndex] = sort(loglambda);
tunp = loglambda(tunpIndex);
loglike = loglike(tunpIndex);
Tlambda = Tlambda(tunpIndex);
tunescore = tunescore(tunpIndex);
numNonzeros = numNonzeros(tunpIndex);
if output == 1
fprintf('\n reg.param  obj              log.likelihood    ');
fprintf('tuning.score      nonzeros\n');
for i=1:length(tunp)
    fprintf(' %10.4e %15.9e  %15.9e   %15.9e  %5d \n', ...
        10^tunp(i), Tlambda(i), loglike(i), tunescore(i),...
        numNonzeros(i));
end
end

else                      % use user sepcified smoothing parameter
    beta_init = 0.0*ones(nbeta,1);
    [tunescore, beta] = ...
        cv(10^loglambda0,beta_init,output);
end
return