function beta = betaoptmt(nmain,loglambda0,bgacv,output)
% Searchs the best tuning parameter and calculates the corresponding beta
% same as in LPS MT
global fBasis;
[sampsize nbeta] = size(fBasis);

if bgacv == 1;
% select the smoothing parameter by bgacv

%--------------------------------------------------------------------------
% The first grid search
%--------------------------------------------------------------------------
d1_1 = 0.15;
d1_2 = 0.1;
nhalf1_1 = 3;
nhalf1_2 = 6;
nhalf2_1 = 3;
nhalf2_2 = 6;
d2_1 = d1_1/nhalf1_1;
d2_2 = d2_1/nhalf2_1;

ngrid1_1 = 2*nhalf1_1 + 1;                  % needs to be odd
ngrid1_2 = 2*nhalf1_2 + 1;
ngrid2_1 = 2*nhalf2_1 + 1;
ngrid2_2 = 2*nhalf2_2 + 1;                  
ngrid = ngrid1_1*ngrid1_2 + ngrid2_1*ngrid2_2;
loglambda1 = (d1_1*nhalf1_1:-d1_1:-d1_1*nhalf1_1) + loglambda0(1);
loglambda2 = (d1_2*nhalf1_2:-d1_2:-d1_2*nhalf1_2) + loglambda0(2);
k=0;
beta_init = 0.0*ones(nbeta,1);   % use starting point of zero at first
tunescore = zeros(1,ngrid);
loglike = zeros(1,ngrid);
Tlambda = zeros(1,ngrid);
numNonzeros = zeros(1,ngrid);
lambdaok = 0;
nsearch = 1;
while ((~lambdaok)&&nsearch<=3)
    lambdaok = 1;
    tunescore_min = Inf;
    for i=1:ngrid1_1
        for j = 1:ngrid1_2
            nctune = (i-1)*ngrid1_1 + j;
            k=k+1;
%            fprintf(' iter%2d, reg.param = %8.5f %8.5f...',k, 10^loglambda1(i),10^loglambda2(j));
            [tunescore(nctune), beta, loglike(nctune), Tlambda(nctune), numNonzeros(nctune)] = ...
                cvmt([10^loglambda1(i),10^loglambda2(j)],nmain,beta_init,output);
            numNonzeros_main = length(find(abs(beta(1:nmain(1)))>1e-6));
            numNonzeros_inter = numNonzeros(nctune)-1-numNonzeros_main;
%            fprintf(' tune.score = %7.4e, nzm =%3d, nzi =%3d\n', tunescore(nctune),...
%                numNonzeros_main,numNonzeros_inter);
            if tunescore(nctune) < tunescore_min,
                tunescore_min = tunescore(nctune);
                loglambda_min = [loglambda1(i) loglambda2(j)];
                numNonzeros_min = numNonzeros(nctune);
                beta_best = beta;
            end
            beta_init = beta;      % remember starting point for next iteration
        end
    end
    nsearch = nsearch + 1;
end
%--------------------------------------------------------------------------
% Reevaluate on a finer grid of regularization parameters near optimal
% value
%--------------------------------------------------------------------------
loglambda21 = loglambda_min(1) + (nhalf2_1*d2_1:-d2_1:-nhalf2_1*d2_1);
loglambda22 = loglambda_min(2) + (nhalf2_2*d2_2:-d2_2:-nhalf2_2*d2_2);
beta_init = beta_best;
for i = 1:ngrid2_1
    for j = 1:ngrid2_2
        k = k + 1;
        nctune = ngrid1_1*ngrid1_2 + (i-1)*ngrid2_1 + j;
 %               fprintf(' iter%2d, reg.param = %8.5f %8.5f...',k, 10^loglambda21(i), 10^loglambda22(j));
        [tunescore(nctune), beta, loglike(nctune), Tlambda(nctune), numNonzeros(nctune)] = ...
            cvmt([10^loglambda21(i) 10^loglambda22(j)],nmain,beta_init,output);
        numNonzeros_main = length(find(abs(beta(1:nmain(1)))>1e-6));
        numNonzeros_inter = numNonzeros(nctune)-1-numNonzeros_main;
 %       fprintf(' tune.score = %7.4e, nzm =%3d, nzi =%3d\n', tunescore(nctune),...
 %               numNonzeros_main,numNonzeros_inter);
        if tunescore(nctune) < tunescore_min,
            tunescore_min = tunescore(nctune);
            loglambda_min = [loglambda21(i) loglambda22(j)];
            numNonzeros_min = numNonzeros(nctune);
            beta_best = beta;
        end
        beta_init = beta;         % remember starting point for next iteration
    end
end
beta = beta_best;

numNonzeros_main = length(find(abs(beta(1:nmain(1)))>1e-6));
numNonzeros_inter = length(find(abs(beta)>1e-6))-1-numNonzeros_main;
if output == 1
fprintf('Minimizing reg.param is %8.5f %8.5f with number of nonzeros %g %g\n\n', ...
    10^loglambda_min(1), 10^loglambda_min(2), numNonzeros_main, numNonzeros_inter);
end

else                            % use user-specified smoothing parameter
    beta_init = 0.0*ones(nbeta,1);
    [tunescore, beta] = ...
        cvmt([10^loglambda0(1) 10^loglambda0(2)],nmain,beta_init,output);
end

return