function [coe stat index] = bgacvp(X, Y)
%bgacvp choose the best subset by backwards selection according to bgacv

[sampsize dim] = size(X);
[bgacv_min coe stat] = calbgacv(X,Y); %the best overall score
index = 1:dim;
nindex = 1:dim;
for i = 1:dim
    reduce = 0;
    isinf = 1;
    bgacv_currmin = Inf; %the best score at this step
    for j = index
        index_curr = setdiff(index,j);
        [bgacv_curr coe_curr stat_curr] = calbgacv(X(:,index_curr),Y);
        if bgacv_curr < bgacv_min
            bgacv_min = bgacv_curr;
            coe = coe_curr;
            stat = stat_curr;
            nindex = index_curr;
            reduce = 1;
        elseif bgacv_curr < bgacv_currmin
            bgacv_currmin = bgacv_curr;
            nindex2 = index_curr;            
            isinf = 0;
        end
    end
    if reduce == 0&&isinf == 0
        index = nindex2;
    elseif reduce == 0&&isinf == 1
        index = setdiff(index,index(1));
    elseif reduce == 1
        index = nindex;
    end
end
index = nindex;
return

function [bgacv coe_curr stat_curr] = calbgacv(X,Y)  %calculate bgacv score
[coe_curr stat_curr] = glmfitbino(X,Y);
if sum(abs(coe_curr))>1e3
    bgacv = Inf;
else
%save tmp1.mat X Y coe_curr stat_curr;
sampsize = length(Y);
B = [ones(sampsize,1) X];
f = B*coe_curr;
ef = exp(f);
sigmasq = ef./(1+ef).^2;
nloglike = sum(-Y.*f + log(1+ef))/sampsize;
U = B'*diag(sigmasq)*B;
%save u1.mat U B f coe_curr;
[EV EU] = eig(U);
eu = diag(EU);
smeig_ind = find(abs(eu)<1e-10);
inv_eu = zeros(length(eu),1);
inv_eu(smeig_ind) = 0;
inv_eu(setdiff(1:length(eu),smeig_ind)) = 1./eu(setdiff(1:length(eu)...
    ,smeig_ind));
H = B*(EV*diag(inv_eu)*EV')*B';
%H = B*inv(U)*B';
bgacv = nloglike + log(sampsize)/2*trace(H)*sum(Y.*(Y-ef./(1+ef)))/...
    sampsize/(sampsize - length(coe_curr));
end
return