function [B inx] = gBasisg(X,nvargroup,i,j,type)
% generate the basis functions for glps, has 5 different types
% type: 1-squares, 2-major triangles, 3-rectangles, 4-big&small triangles,
% 5-small triangle
if i>=j
    fprintf('i must be smaller than j\n')
    return
end
[sampsize dim] = size(X);
ngroup1 = floor(dim/nvargroup);
inx1 = nvargroup*(i-1)+1:nvargroup*i;
inx2 = nvargroup*(j-1)+1:nvargroup*j;
kk = 1;

if type == 1                                   % Squares
    inx.main = [inx1 inx2]';
    B_main = [X(:,inx1) X(:,inx2)];
    B_inter = zeros(sampsize,nvargroup^2);
    inx.inter = zeros(nvargroup^2,2);
    for i = inx1
        for j = inx2
            B_inter(:,kk) = X(:,i).*X(:,j);
            inx.inter(kk,:) = [i j];
            kk = kk + 1;
        end
    end
elseif type == 2                               % Major triangles
    inx.main = [inx1 inx2]';
    B_main = [X(:,inx1) X(:,inx2)];
    B_inter = zeros(sampsize,nvargroup*(nvargroup-1));
    inx.inter = zeros(nvargroup*(nvargroup-1),2);
    for i = inx1
        for j = i+1:inx1(nvargroup)
            B_inter(:,kk) = X(:,i).*X(:,j);
            inx.inter(kk,:) = [i j];
            kk = kk + 1;
        end
    end
    for i = inx2
        for j = i+1:inx2(nvargroup)
            B_inter(:,kk) = X(:,i).*X(:,j);
            inx.inter(kk,:) = [i j];
            kk = kk + 1;
        end
    end
elseif type == 3                               % Rectangles
    j = ngroup1 + 1;
    inx2 = nvargroup*(j-1)+1:dim;
    B_main = [X(:,inx1) X(:,inx2)];
    inx.main = [inx1 inx2]';
    B_inter = zeros(sampsize,nvargroup*(dim - ngroup1*nvargroup));
    inx.inter = zeros(nvargroup*(dim - ngroup1*nvargroup),2);
    for i = inx1
        for j = inx2
            B_inter(:,kk) = X(:,i).*X(:,j);
            inx.inter(kk,:) = [i j];
            kk = kk + 1;
        end
    end
elseif type == 4                               % big and small triangles
    i = ngroup1;
    j = ngroup1 + 1;
    inx1 = nvargroup*(i-1)+1:nvargroup*i;
    inx2 = nvargroup*(j-1)+1:dim;
    leninx2 = length(inx2);
    B_main = [X(:,inx1) X(:,inx2)];
    inx.main = [inx1 inx2]';
    B_inter = zeros(sampsize,nvargroup*(nvargroup-1)/2 + ...
        leninx2*(leninx2-1)/2);
    inx.inter = zeros(size(B_inter,2),2);
    for i = inx1
        for j = i+1:inx1(length(inx1))
            B_inter(:,kk) = X(:,i).*X(:,j);
            inx.inter(kk,:) = [i j];
            kk = kk + 1;
        end
    end                                        % small triangles only
    for i = inx2
        for j = i+1:inx2(length(inx2))
            B_inter(:,kk) = X(:,i).*X(:,j);
            inx.inter(kk,:) = [i j];
            kk = kk + 1;
        end
    end
else
    i = ngroup1 + 1;
%    j = ngroup1 + 1;
    inx1 = nvargroup*(i-1)+1:dim;
    leninx1 = length(inx1);
    B_main = X(:,inx1);
    inx.main = inx1';
    B_inter = zeros(sampsize,leninx1*(leninx1-1)/2);
    inx.inter = zeros(size(B_inter,2),2);
    for i = inx1
        for j = i+1:inx1(length(inx1))
            B_inter(:,kk) = X(:,i).*X(:,j);
            inx.inter(kk,:) = [i j];
            kk = kk + 1;
        end
    end
end
B = [B_main B_inter ones(sampsize,1)];
return


