function [bb,stats] = glmfitbino(x,y)
%GLMFITBINO Fit a generalized linear model for binomial distribution.


n = size(x,1); % get the size of x
N = ones(n,1);

x = [ones(size(x,1),1) x];
dataClass = superiorfloat(x,y);
x = cast(x,dataClass);
y = cast(y,dataClass);
sumx = sum(x);
[ordsumx indsumx] = sort(sumx); 
ncolx = size(x,2);

% If x is rank deficient (perhaps because it is overparameterized), we will
% warn and remove columns, and the corresponding coefficients and std. errs.
% will be forced to zero.
[Q,R] = qr(x,0);
perm = 1:ncolx;
rankx = sum(abs(diag(R)) > max(abs(diag(R)))*max(n,ncolx)*eps(class(R)));

if rankx < ncolx
    rankxdef = ncolx - rankx;
    perm = setdiff(perm,indsumx(1:rankxdef));
    nx = x(:,perm);
    [Q,R] = qr(nx,0);
    ranknx = sum(abs(diag(R)) > max(abs(diag(R)))*max(n,size(nx,2))*eps(class(R)));
    diagR = diag(R);
    [orddiagR, inddiagR] = sort(abs(diagR));
    if ranknx < size(nx,2);
        ranknxdef = size(nx,2) - ranknx;
        rankxdef = rankxdef + ranknxdef;
        x = nx(:,sort(inddiagR(1+ranknxdef:size(nx,2))));
%    length(perm)
        perm = perm(setdiff(1:length(perm),inddiagR(1:ranknxdef)));
        [newordsumx newindsumx] = sort(sum(x));
        indsumx = [ones(1,rankxdef) perm(newindsumx)];
        clear newordersumx newindsumx
%    length(perm)
    else
        x = nx;
%    size(x)
    end
else
    rankxdef = 0;
end

% Number of observations after removing missing data, number of coeffs after
% removing dependent cols and (possibly) adding a constant term.
[n,p] = size(x);

pwts = 1; %set the weight to be 1


% Define variance and deviance for binomial, now that N has NaNs removed.
sqrtN = sqrt(N);
sqrtvarFun = @(mu) sqrt(mu).*sqrt(1-mu) ./ sqrtN;
devFun = @(mu,y) 2*N.*(y.*log((y+(y==0))./mu) + (1-y).*log((1-y+(y==1))./(1-mu)));

% Instantiate functions for one of the canned links, or validate a
% user-defined link specification.
muLims = [eps(dataClass) 1-eps(dataClass)];
etaLims = log(muLims./(1-muLims));

linkFun = @(mu) log(mu ./ (1-mu));
dlinkFun = @(mu) 1 ./ (mu .* (1-mu));
ilinkFun = @(eta) 1 ./ (1 + exp(-min(max(eta,etaLims(1)),etaLims(2))));


% Initialize mu and eta from y.
mu = (N .* y + 0.5) ./ (N + 1);
eta = linkFun(mu);

% Set up for iterations
iter = 0;
iterLim = 100;
seps = sqrt(eps);
convcrit = 1e-6;
b = zeros(p,1,dataClass);


while iter <= iterLim
    iter = iter+1;

    % Compute adjusted dependent variable for least squares fit
    deta = dlinkFun(mu);
    z = eta + (y - mu) .* deta;

    % Compute IRLS weights the inverse of the variance function
    sqrtw = sqrt(pwts) ./ (abs(deta) .* sqrtvarFun(mu));


    % Compute coefficient estimates for this iteration - the IRLS step
    b_old = b;
    [b,R] = wfit(z, x, sqrtw);

    % Form current linear predictor
    eta = x * b;

    % Compute predicted mean using inverse link function
    mu = ilinkFun(eta);

    % Force mean in bounds, in case the link function is a wacky choice
    if any(mu < muLims(1) | muLims(2) < mu)
        mu = max(min(mu,muLims(2)),muLims(1));
    end

    % Check stopping conditions
    if (~any(abs(b-b_old) > convcrit * max(seps, abs(b_old)))), break; end
    if iter == iterLim
        rankxdef = rankxdef + 1;
        sumx = sum(x);
        perm = setdiff(perm,indsumx(rankxdef));
        [newordsumx newindsumx] = sort(sumx); 
        x = x(:,setdiff(1:p,newindsumx(1)));
        setdiff(1:p,newindsumx(1));
        p = p - 1;
        iter = 1;
        b = zeros(p,1,dataClass);
        mu = (N .* y + 0.5) ./ (N + 1);
        eta = linkFun(mu);
        
    end
end

bb = zeros(ncolx,1,dataClass); 
%ncolx
%perm
%b
bb(perm) = b;

if nargout > 1
    % Compute the sum of squares used to estimate dispersion, and the
    % Anscombe residuals.
    ssr = sum(pwts .* (y - mu).^2 ./ (mu .* (1 - mu) ./ N));

    dfe = n - p;
    stats.beta = bb;
    stats.dfe = dfe;
    stats.sfit = sqrt(ssr/dfe);
    
    % Find coefficient standard errors and correlations
    RI = R\eye(p);
    C = RI * RI';
    se = sqrt(diag(C));
    C = C ./ (se * se');
    stats.se = zeros(ncolx,1,dataClass); stats.se(perm) = se;
    stats.coeffcorr = zeros(ncolx,ncolx,dataClass); stats.coeffcorr(perm,perm) = C;
    stats.t = NaN(ncolx,1,dataClass); 
    stats.t(perm) = b ./ se;
    stats.p = 2 * normcdf(-abs(stats.t));
    stats.p(setdiff(1:ncolx,perm)) = 1;
end


function [b,R] = wfit(y,x,sw)
% Perform a weighted least squares fit
[n,p] = size(x);
yw = y .* sw;
xw = x .* sw(:,ones(1,p));
% No pivoting, no basic solution.  We've removed dependent cols from x, and
% checked the weights, so xw should be full rank.
[Q,R] = qr(xw,0);
b = R \ (Q'*yw);
